public inbox for linux-kernel@vger.kernel.org
 help / color / mirror / Atom feed
From: Steven Rostedt <rostedt@goodmis.org>
To: linux-kernel@vger.kernel.org
Cc: Ingo Molnar <mingo@elte.hu>,
	Andrew Morton <akpm@linux-foundation.org>,
	Frederic Weisbecker <fweisbec@gmail.com>,
	Vaibhav Nagarnaik <vnagarnaik@google.com>,
	Ingo Molnar <mingo@redhat.com>, Michael Rubin <mrubin@google.com>,
	David Sharp <dhsharp@google.com>,
	Linus Torvalds <torvalds@linux-foundation.org>
Subject: [PATCH 01/15] tracing: Schedule a delayed work to call wakeup()
Date: Thu, 09 Jun 2011 13:27:45 -0400	[thread overview]
Message-ID: <20110609172910.425793191@goodmis.org> (raw)
In-Reply-To: 20110609172744.333794089@goodmis.org

[-- Attachment #1: 0001-tracing-Schedule-a-delayed-work-to-call-wakeup.patch --]
[-- Type: text/plain, Size: 4592 bytes --]

From: Vaibhav Nagarnaik <vnagarnaik@google.com>

In using syscall tracing by concurrent processes, the wakeup() that is
called in the event commit function causes contention on the spin lock
of the waitqueue. I enabled sys_enter_getuid and sys_exit_getuid
tracepoints, and by running getuid_microbench from autotest in parallel
I found that the contention causes exponential latency increase in the
tracing path.

The autotest binary getuid_microbench calls getuid() in a tight loop for
the given number of iterations and measures the average time required to
complete a single invocation of syscall.

The patch schedules a delayed work after 2 ms once an event commit calls
to wake up the trace wait_queue. This removes the delay caused by
contention on spin lock in wakeup() and amortizes the wakeup() calls
scheduled over the 2 ms period.

In the following example, the script enables the sys_enter_getuid and
sys_exit_getuid tracepoints and runs the getuid_microbench in parallel
with the given number of processes. The output clearly shows the latency
increase caused by contentions.

$ ~/getuid.sh 1
1000000 calls in 0.720974253 s (720.974253 ns/call)

$ ~/getuid.sh 2
1000000 calls in 1.166457554 s (1166.457554 ns/call)
1000000 calls in 1.168933765 s (1168.933765 ns/call)

$ ~/getuid.sh 3
1000000 calls in 1.783827516 s (1783.827516 ns/call)
1000000 calls in 1.795553270 s (1795.553270 ns/call)
1000000 calls in 1.796493376 s (1796.493376 ns/call)

$ ~/getuid.sh 4
1000000 calls in 4.483041796 s (4483.041796 ns/call)
1000000 calls in 4.484165388 s (4484.165388 ns/call)
1000000 calls in 4.484850762 s (4484.850762 ns/call)
1000000 calls in 4.485643576 s (4485.643576 ns/call)

$ ~/getuid.sh 5
1000000 calls in 6.497521653 s (6497.521653 ns/call)
1000000 calls in 6.502000236 s (6502.000236 ns/call)
1000000 calls in 6.501709115 s (6501.709115 ns/call)
1000000 calls in 6.502124100 s (6502.124100 ns/call)
1000000 calls in 6.502936358 s (6502.936358 ns/call)

After the patch, the latencies scale better.
1000000 calls in 0.728720455 s (728.720455 ns/call)

1000000 calls in 0.842782857 s (842.782857 ns/call)
1000000 calls in 0.883803135 s (883.803135 ns/call)

1000000 calls in 0.902077764 s (902.077764 ns/call)
1000000 calls in 0.902838202 s (902.838202 ns/call)
1000000 calls in 0.908896885 s (908.896885 ns/call)

1000000 calls in 0.932523515 s (932.523515 ns/call)
1000000 calls in 0.958009672 s (958.009672 ns/call)
1000000 calls in 0.986188020 s (986.188020 ns/call)
1000000 calls in 0.989771102 s (989.771102 ns/call)

1000000 calls in 0.933518391 s (933.518391 ns/call)
1000000 calls in 0.958897947 s (958.897947 ns/call)
1000000 calls in 1.031038897 s (1031.038897 ns/call)
1000000 calls in 1.089516025 s (1089.516025 ns/call)
1000000 calls in 1.141998347 s (1141.998347 ns/call)

Signed-off-by: Vaibhav Nagarnaik <vnagarnaik@google.com>
Cc: Frederic Weisbecker <fweisbec@gmail.com>
Cc: Ingo Molnar <mingo@redhat.com>
Cc: Michael Rubin <mrubin@google.com>
Cc: David Sharp <dhsharp@google.com>
Cc: Linus Torvalds <torvalds@linux-foundation.org>
Link: http://lkml.kernel.org/r/1305059241-7629-1-git-send-email-vnagarnaik@google.com
Signed-off-by: Steven Rostedt <rostedt@goodmis.org>
---
 kernel/trace/trace.c |   23 ++++++++++++-----------
 1 files changed, 12 insertions(+), 11 deletions(-)

diff --git a/kernel/trace/trace.c b/kernel/trace/trace.c
index ee9c921..2af132e 100644
--- a/kernel/trace/trace.c
+++ b/kernel/trace/trace.c
@@ -343,26 +343,27 @@ unsigned long trace_flags = TRACE_ITER_PRINT_PARENT | TRACE_ITER_PRINTK |
 static int trace_stop_count;
 static DEFINE_SPINLOCK(tracing_start_lock);
 
+static void wakeup_work_handler(struct work_struct *work)
+{
+	wake_up(&trace_wait);
+}
+
+static DECLARE_DELAYED_WORK(wakeup_work, wakeup_work_handler);
+
 /**
  * trace_wake_up - wake up tasks waiting for trace input
  *
  * Simply wakes up any task that is blocked on the trace_wait
  * queue. These is used with trace_poll for tasks polling the trace.
  */
 void trace_wake_up(void)
 {
-	int cpu;
+	const unsigned long delay = msecs_to_jiffies(2); //delay 2 ms
 
 	if (trace_flags & TRACE_ITER_BLOCK)
 		return;
-	/*
-	 * The runqueue_is_locked() can fail, but this is the best we
-	 * have for now:
-	 */
-	cpu = get_cpu();
-	if (!runqueue_is_locked(cpu))
-		wake_up(&trace_wait);
-	put_cpu();
+	schedule_delayed_work(&wakeup_work, delay);
 }
 
 static int __init set_buf_size(char *str)
@@ -343,26 +343,27 @@ unsigned long trace_flags = TRACE_ITER_PRINT_PARENT | TRACE_ITER_PRINTK |
  * trace_wake_up - wake up tasks waiting for trace input
  *
- * Simply wakes up any task that is blocked on the trace_wait
- * queue. These is used with trace_poll for tasks polling the trace.
+ * Schedules a delayed work to wake up any task that is blocked on the
+ * trace_wait queue. These is used with trace_poll for tasks polling the
+ * trace.
  */
 void trace_wake_up(void)
-- 
1.7.4.4



  reply	other threads:[~2011-06-09 17:29 UTC|newest]

Thread overview: 41+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2011-06-09 17:27 [PATCH 00/15] [GIT PULL] tracing: various updates Steven Rostedt
2011-06-09 17:27 ` Steven Rostedt [this message]
2011-06-13 10:07   ` [PATCH 01/15] tracing: Schedule a delayed work to call wakeup() Ingo Molnar
2011-06-13 10:27     ` Pekka Enberg
2011-06-13 11:45       ` Steven Rostedt
2011-06-09 17:27 ` [PATCH 02/15] tracing: Use NUMA allocation for per-cpu ring buffer pages Steven Rostedt
2011-06-13 10:09   ` Ingo Molnar
2011-06-13 11:28     ` Steven Rostedt
2011-06-09 17:27 ` [PATCH 03/15] tracing: Add a free on close control mechanism for buffer_size_kb Steven Rostedt
2011-06-13 10:12   ` Ingo Molnar
2011-06-13 11:39     ` Steven Rostedt
2011-06-13 11:49       ` Ingo Molnar
2011-06-13 11:54         ` Steven Rostedt
2011-06-13 19:12           ` Vaibhav Nagarnaik
2011-06-13 20:01           ` Vaibhav Nagarnaik
2011-06-14  0:37             ` Steven Rostedt
2011-06-14  0:43               ` Vaibhav Nagarnaik
2011-06-09 17:27 ` [PATCH 04/15] ftrace: Fixed an include coding style issue Steven Rostedt
2011-06-09 17:27 ` [PATCH 05/15] async: " Steven Rostedt
2011-06-09 17:27 ` [PATCH 06/15] tracing, function_graph: Remove dependency of abstime and duration Steven Rostedt
2011-06-09 17:27 ` [PATCH 07/15] tracing, function_graph: Merge overhead and duration display Steven Rostedt
2011-06-09 17:27 ` [PATCH 08/15] tracing, function: Fix trace header to follow context-info option Steven Rostedt
2011-06-09 17:27 ` [PATCH 09/15] tracing, function_graph: Remove lock-depth from latency trace Steven Rostedt
2011-06-09 17:27 ` [PATCH 10/15] tracing, function_graph: Add context-info support for function_graph Steven Rostedt
2011-06-09 17:27 ` [PATCH 11/15] tracing: Convert to kstrtoul_from_user Steven Rostedt
2011-06-09 17:27 ` [PATCH 12/15] ring-buffer: Set __GFP_NORETRY flag for ring buffer allocating Steven Rostedt
2011-06-09 17:27 ` [PATCH 13/15] x86: Swap save_stack_trace_regs parameters Steven Rostedt
2011-06-13 10:18   ` Ingo Molnar
2011-06-13 11:43     ` Steven Rostedt
2011-06-09 17:27 ` [PATCH 14/15] stack_trace: Add weak save_stack_trace_regs() Steven Rostedt
2011-06-13 10:19   ` Ingo Molnar
2011-06-13 10:52     ` Masami Hiramatsu
2011-06-13 11:42       ` Ingo Molnar
2011-06-09 17:27 ` [PATCH 15/15] tracing/kprobes: Fix kprobe-tracer to support stack trace Steven Rostedt
2011-06-13 10:21   ` Ingo Molnar
2011-06-13 11:44     ` Steven Rostedt
2011-06-13 11:50       ` Ingo Molnar
2011-06-13 12:14         ` Steven Rostedt
2011-06-14  1:25           ` Masami Hiramatsu
2011-06-14  2:08             ` Steven Rostedt
2011-06-14 11:22               ` Peter Zijlstra

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20110609172910.425793191@goodmis.org \
    --to=rostedt@goodmis.org \
    --cc=akpm@linux-foundation.org \
    --cc=dhsharp@google.com \
    --cc=fweisbec@gmail.com \
    --cc=linux-kernel@vger.kernel.org \
    --cc=mingo@elte.hu \
    --cc=mingo@redhat.com \
    --cc=mrubin@google.com \
    --cc=torvalds@linux-foundation.org \
    --cc=vnagarnaik@google.com \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox